@@ -1,6 +1,6 @@
 from PySide2 import QtCore, QtGui, QtWidgets
 from PySide2.QtWidgets import (QGroupBox, QFormLayout, QLabel, QLineEdit, QVBoxLayout, QWidget, QMainWindow)
-from PySide2.QtCore import Qt, Signal, Slot, QPoint
+from PySide2.QtCore import Qt, Signal, Slot, QPoint, QEvent
 from PySide2.QtGui import QPainter, QBrush, QPen, QPixmap, QColor
 from typing import Union, Tuple
 import os
@@ -60,6 +60,7 @@ class ClickableScene(QtWidgets.QGraphicsScene):
     def mouseReleaseEvent(self, event):
         self.release.emit(event)
         super().mouseReleaseEvent(event)
+
         
 
 class VideoFrame(QtWidgets.QGraphicsView):
@@ -92,9 +93,27 @@ class VideoFrame(QtWidgets.QGraphicsView):
             self.initialize_video(videoFile)
             self.update()
         self.setStyleSheet("background:transparent;")
-        
+        self.setMouseTracking(True)    
+
+        # for pan/zoom
+        self.resize_on_each_frame = True
+        self.grabGesture(Qt.PinchGesture)
+        self.setResizeAnchor(QtWidgets.QGraphicsView.AnchorUnderMouse)
+
+    def event(self, event):
+        out = super(VideoFrame, self).event(event)
+        if type(event)==QtWidgets.QGestureEvent:
+            gesture = event.gesture(Qt.PinchGesture)
+            scale = gesture.scaleFactor()
+            last_scale = gesture.lastScaleFactor()
+            self.scale(scale, last_scale)
+        return out
     
     def initialize_image(self, imagefile: Union[str, os.PathLike])    : 
+        if self.vid is not None:
+            self.vid.close()
+            self.vid = None
+
         self.videofile = imagefile
         assert os.path.isfile(imagefile)
         
@@ -111,20 +130,31 @@ class VideoFrame(QtWidgets.QGraphicsView):
         # print('new fnum: {}'.format(self.current_fnum))
         self.show_image(self.frame)
         self.frameNum.emit(self.current_fnum)
+        self.fitInView()
 
         # print(self.palette())
 
     def initialize_video(self, videofile: Union[str, os.PathLike]):
         if self.vid is not None:
             self.vid.close()
-            # if hasattr(self.vid, 'cap'):
-            #     self.vid.cap.release()
+            self.vid = None
+
         self.videofile = videofile
         self.vid = VideoReader(videofile)
         # self.frame = next(self.vid)
         self.initialized.emit(len(self.vid))
         # there was a bug where sometimes subsequent videos with the same frame would not update the image
         self.update_frame(0, force_update=True)
+        self.fitInView()
+
+    def get_image_names(self):
+        if self.vid is None: # single image
+            return [self.videofile]
+        elif isinstance(self.vid.file_object, list): # image directory 
+            return [os.path.split(n)[1] for n in self.vid.file_object]
+        else: # video
+            return [self.videofile]*len(self.vid)
+        
 
     def update_frame(self, value, force_update: bool=False):
         # print('updating')
@@ -154,6 +184,7 @@ class VideoFrame(QtWidgets.QGraphicsView):
         # print('new fnum: {}'.format(self.current_fnum))
         self.show_image(self.frame)
         self.frameNum.emit(self.current_fnum)
+        if self.resize_on_each_frame: self.fitInView()
         
     def next_frame(self):
         self.update_frame(self.current_fnum + 1)
@@ -195,13 +226,13 @@ class VideoFrame(QtWidgets.QGraphicsView):
         # THIS LINE CHANGES THE SCENE WIDTH AND HEIGHT
         self._photo.setPixmap(qpixmap)
 
-        self.fitInView()
+        if self.resize_on_each_frame: self.fitInView()
         self.update()
         # self.show()
 
-    def resizeEvent(self, event):
-        if hasattr(self, 'vid'):
-            self.fitInView()
+    # def resizeEvent(self, event):
+    #     if hasattr(self, 'vid'):
+    #         self.fitInView()
 
 
 class ScrollbarWithText(QtWidgets.QWidget):
@@ -405,11 +436,10 @@ class KeypointGroup(QtWidgets.QWidget):
     selected = Signal(int)
     data = Signal(dict)
     
-    def __init__(self, keypoint_dict, scene, parent=None, colormap:str='viridis', radius=20):
+    def __init__(self, keypoint_dict, scene, parent=None, colormap:str='viridis', radius=20, 
+                 text_over_mouse=True, click_type_to_add_keypoint='right'):
         super().__init__(parent)
         
-        print('constructor')
-        
         self.cmap = plt.get_cmap(colormap)
         
         colors = plt.get_cmap(colormap)(np.linspace(0, 1, len(keypoint_dict)))
@@ -425,12 +455,15 @@ class KeypointGroup(QtWidgets.QWidget):
         self.radius = radius
         self.add_to_scene(scene)
         self.index = 0
+        self.key = None
         self.selected.emit(self.index)
         self.scene = scene
         self.tmp_selected = None
-        
+        self.text_over_mouse = text_over_mouse
+        self.click_type_to_add_keypoint = click_type_to_add_keypoint
         self.set_data(keypoint_dict)
-        
+        self.text = None
+        self.update_text()
         # self.data = {}
         # self.scene.viewport().installEventFilter(self)
             
@@ -451,12 +484,12 @@ class KeypointGroup(QtWidgets.QWidget):
                 self.keypoints[key].set_coords(*value)
             
     def add_to_scene(self, scene):
-        print('add to scene')
+        # print('add to scene')
         for key, value in self.keypoints.items():
             scene.addItem(value)
             
     def remove_from_scene(self):
-        print('remove from scene')
+        # print('remove from scene')
         for key, value in self.keypoints.items():
             self.scene.removeItem(value)
             
@@ -491,9 +524,27 @@ class KeypointGroup(QtWidgets.QWidget):
             # don't do anything. should make sure that there isn't a loop between various widgets that control this
             return
         self.index = index
+        self.update_text()
         self.selected.emit(self.index)
     
-    def right_click(self, event):
+    def update_text(self):
+        self.key = self.keys[self.index]
+        if not self.text_over_mouse:
+            return
+        color = self.colors[self.index]
+        line_color = QColor(color[0], color[1], color[2], color[3])
+        face_color = QColor(color[0], color[1], color[2], int(color[3]*0.3))
+        pen = QPen(line_color, 0.5, Qt.SolidLine, Qt.FlatCap, Qt.MiterJoin)
+        brush = QBrush(face_color)
+        if self.text is None:
+            self.text = QtWidgets.QGraphicsSimpleTextItem(self.key)
+            self.scene.addItem(self.text)
+
+        self.text.setText(self.key)
+        self.text.setBrush(brush)
+        self.text.setPen(pen)
+    
+    def add_keypoint(self, event):
         pos = event.scenePos()
         x, y = pos.x(), pos.y()
         
@@ -505,7 +556,7 @@ class KeypointGroup(QtWidgets.QWidget):
         # print(x,y)
         self.set_selected(self.index+1)
         
-    def left_click(self, event):
+    def move_keypoint(self, event):
         pos = event.scenePos()
         x, y = pos.x(), pos.y()
         dists = self.get_distance_to_keypoints(x, y)
@@ -541,22 +592,34 @@ class KeypointGroup(QtWidgets.QWidget):
     @Slot(QtGui.QMouseEvent)
     def receive_click(self, event):
         if event.button() == QtCore.Qt.RightButton:
-            self.right_click(event)
+            if self.click_type_to_add_keypoint == 'left':
+                self.move_keypoint(event)
+            else:
+                self.add_keypoint(event)
+
         elif event.button() == QtCore.Qt.LeftButton:
-            self.left_click(event)
+            if self.click_type_to_add_keypoint == 'left':
+                self.add_keypoint(event)
+            else:
+                self.move_keypoint(event)
     
     @Slot(QtGui.QMouseEvent)
     def receive_move(self, event):
+        pos = event.scenePos()
+        x, y = pos.x(), pos.y()
+        # print(x,y)
         # print(event.buttons())
-        if event.buttons() == QtCore.Qt.LeftButton:
-            # print(self.tmp_selected)
+        if ((self.click_type_to_add_keypoint != 'left' and event.buttons() == QtCore.Qt.LeftButton) or
+            (self.click_type_to_add_keypoint == 'left' and event.buttons() == QtCore.Qt.RightButton)):
             if self.tmp_selected is None:
                 return
-            pos = event.scenePos()
-            x, y = pos.x(), pos.y()
-            
+
             self.keypoints[self.keys[self.tmp_selected]].set_coords(x, y, self.radius)
             self.broadcast_data()
+        if self.text_over_mouse:
+            if self.text is not None:
+                # print(x, y)
+                self.text.setPos(x + 10, y + 10)
             # print(event.scenePos())
     
     @Slot(QtGui.QMouseEvent)
@@ -640,26 +703,6 @@ class KeypointButtons(QtWidgets.QWidget):
         self.selected.emit(self.index)
 
         
-class MainWindow(QtWidgets.QMainWindow):
-    def __init__(self, parent=None):
-        super(MainWindow, self).__init__(parent)
-
-        widget = VideoPlayer(videoFile=r'/media/jim/DATA_SSD/armo/dataset_for_labeling/images/SS21_190508_140054_002526')
-        
-        kp = {'nose': (50, 50, 5), 
-              'forepaw': (100,100,5), 
-              'hindpaw': []}
-        
-        keypoints = KeypointGroup(kp, scene=widget.scene, parent=widget, radius=5)
-        widget.scene.click.connect(keypoints.receive_click)
-        widget.scene.move.connect(keypoints.receive_move)
-        widget.scene.release.connect(keypoints.receive_release)
-        # keypoint = Keypoint(100, 100, parent=widget)
-        # widget.scene.addItem(keypoint)
-        # scene =CroppingOverlay(self)
-        # view = QtWidgets.QGraphicsView(scene)
-        self.setCentralWidget(widget)
-
 
 def simple_popup_question(parent, message: str):
     # message = 'You have unsaved changes. Are you sure you want to quit?'
@@ -667,20 +710,3 @@ def simple_popup_question(parent, message: str):
                                            message, QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
     return reply == QtWidgets.QMessageBox.Yes
 
-
-if __name__ == '__main__':
-    app = QtWidgets.QApplication([])
-    # testing = VideoPlayer(videoFile=r'/home/jb/Downloads/Basler_acA1300-200um__22273960__20200120_113922411.mp4')
-    testing = MainWindow()
-    testing.resize(640, 480)
-    # volume = VideoPlayer(r'C:\DATA\mouse_reach_processed\M134_20141203_v001.h5')
-    # testing = QtWidgets.QMainWindow()
-    # testing.initialize(n=6, n_timepoints=15000, debug=True)
-    # testing = ShouldRunInference(['M134_20141203_v001',
-    #                               'M134_20141203_v002',
-    #                               'M134_20141203_v004'],
-    #                              [True, True, False])
-    # testing = MainWindow()
-    # testing.setMaximumHeight(250)
-    testing.show()
-    app.exec_()
